searchData={"items":[{"type":"module","title":"Kubereq","doc":"A Kubernetes client for Elixir based on `Req`.","ref":"Kubereq.html"},{"type":"module","title":"Usage - Kubereq","doc":"First, attach `kubereq` to your `Req` request (see `attach/2` for options):\n\n    Req.new() |> Kubereq.attach()\n\nNow you can use plain Req functionality. However, the functions defined in\nthis module make it much easier to perform the most common operation.","ref":"Kubereq.html#module-usage"},{"type":"module","title":"Usage with plain Req functionality - Kubereq","doc":"Use `Kubereq.Kubeconfig.Default` to create connection to cluster and\nplain `Req.request()` to make the request\n\n```\nreq = Req.new() |> Kubereq.attach()\n\nReq.request!(req,\n  api_version: \"v1\",\n  kind: \"ServiceAccount\",\n  operation: :get,\n  path_params: [namespace: \"default\", name: \"default\"]\n)\n```\n\nYou can pass your own Kubeconfigloader pipeline when attaching:\n\n```\nreq = Req.new() |> Kubereq.attach(kubeconfig: {Kubereq.Kubeconfig.File, path: \"/path/to/kubeconfig.yaml\"})\n\nReq.request!(req,\n  api_version: \"v1\",\n  kind: \"ServiceAccount\",\n  operation: :get,\n  path_params: [namespace: \"default\", name: \"default\"]\n)\n```\n\nPrepare a `Req` struct for a specific resource:\n\n```\nsa_req = Req.new() |> Kubereq.attach(api_version: \"v1\", kind: \"ServiceAccount\")\n\nReq.request!(sa_req,  operation: :get, path_params: [namespace: \"default\", name: \"default\"])\nReq.request!(sa_req,  operation: :list, path_params: [namespace: \"default\"])\n```","ref":"Kubereq.html#module-usage-with-plain-req-functionality"},{"type":"module","title":"Kubereq API - Kubereq","doc":"While this library can attach to any `Req` struct, it is sometimes easier\nto prepare `Req` for a specific resource and then use the functions\ndefined in the `Kubereq` module.\n\n```\nsa_req = Req.new() |> Kubereq.attach(api_version: \"v1\", kind: \"ServiceAccount\")\n\nKubereq.get(sa_req, \"my-namespace\", \"default\")\nKubereq.list(sa_req, \"my-namespace\")\n```\n\nOr use the functions right away, defining the resource through options:\n\n```\nreq = Req.new() |> Kubereq.attach()\n\nKubereq.get(req, \"my-namespace\", \"default\", api_version: \"v1\", kind: \"ServiceAccount\")\n\n# get the \"status\" subresource of the default namespace\nKubereq.get(req, \"my-namespace\", api_version: \"v1\", kind: \"Namespace\", subresource: \"status\")\n```\n\nFor resources defined by Kubernetes, the `api_version` can be omitted:\n\n```\nReq.new()\n|> Kubereq.attach(kind: \"Namespace\")\n|> Kubereq.get(\"my-namespace\")\n```","ref":"Kubereq.html#module-kubereq-api"},{"type":"module","title":"Options - Kubereq","doc":"`kubereq` registeres the following options with `Req`:\n\n  * `:kubeconfig` - A `%Kubereq.Kubeconfig{}` struct. The `attach/2` function also accepts a\n    Kubeconf pipeline (e.g. `Kubereq.Kubeconfig.Default`)\n  * `:api_version` - The group and version of the targeted resource (case sensitive)\n  * `:kind` - The kind of the targeted resource (case sensitive)\n  * `:resource_path` - Can be defined instead of `:api_version` and `:kind`. The path to the\n    targeted resource with placeholders for `:namespace` and `:name`\n    (e.g. `api/v1/namespaces/:namespace/configmaps/:name`)\n  * `:field_selectors` - See `Kubereq.Step.FieldSelector`\n  * `:label_selectors` - See `Kubereq.Step.LabelSelector`\n  * `:operation` - The operation on the resource (one of `:create`, `:get` `:update`,\n    `:delete`, `:delete_all`, `:apply`, `:json_patch`, `:merge_patch`, `:watch`)\n  * `:subresource` - Some operations can be performed on subresources\n    (e.g. `status` or `scale`)","ref":"Kubereq.html#module-options"},{"type":"function","title":"Kubereq.apply/5","doc":"Applies the given `resource` using a Server-Side-Apply Patch. Returns a\nresponse or an error.\n\nSee the [documentation](https://kubernetes.io/docs/reference/using-api/server-side-apply/)\nfor a documentation on `field_manager` and `force` arguments.","ref":"Kubereq.html#apply/5"},{"type":"function","title":"Examples - Kubereq.apply/5","doc":"Req.new()\n    |> Kubereq.attach(api_version: \"v1\", kind: \"ConfigMap\")\n    |> Kubereq.apply(resource)","ref":"Kubereq.html#apply/5-examples"},{"type":"function","title":"Kubereq.apply!/5","doc":"Applies the given `resource` using a Server-Side-Apply Patch. Returns a\nresponse or raises an error.\n\nSee the [documentation](https://kubernetes.io/docs/reference/using-api/server-side-apply/)\nfor a documentation on `field_manager` and `force` arguments.","ref":"Kubereq.html#apply!/5"},{"type":"function","title":"Examples - Kubereq.apply!/5","doc":"Req.new()\n    |> Kubereq.attach(api_version: \"v1\", kind: \"ConfigMap\")\n    |> Kubereq.apply!(resource)","ref":"Kubereq.html#apply!/5-examples"},{"type":"function","title":"Kubereq.attach/2","doc":"Attaches `kubereq` to a `Req.Request` struct for making HTTP requests to a\nKubernetes cluster. You can optionally pass a Kubernetes configuration or\npipeline via `kubeconfig` option. If it is omitted, the default config\n`Kubereq.Kubeconfig.Default` is loaded.","ref":"Kubereq.html#attach/2"},{"type":"function","title":"Examples - Kubereq.attach/2","doc":"Req.new() |> Kubereq.attach()","ref":"Kubereq.html#attach/2-examples"},{"type":"function","title":"Options - Kubereq.attach/2","doc":"All options (see Options section in module doc) are accepted and merged with\nthe given req.","ref":"Kubereq.html#attach/2-options"},{"type":"function","title":"Kubereq.can_i?/3","doc":"Checks whether the authenticated user is authorized to perform a specific\naction.\n\nCreates a [SelfSubjectAccessReview][SelfSubjectAccessReview] resource with\nthe given `attributes` and sends it to the API Server. It returns\n`.status.allowed` from the result (boolean). In case of an error, the\nfunction returns `false`.","ref":"Kubereq.html#can_i?/3"},{"type":"function","title":"Attributes - Kubereq.can_i?/3","doc":"`attributes` is a Keyword list that allows the following keywords (See\nattribute descriptions on the  [Kubernetes documentation][SelfSubjectAccessReview])\n\n[SelfSubjectAccessReview]: https://kubernetes.io/docs/reference/kubernetes-api/authorization-resources/self-subject-access-review-v1/","ref":"Kubereq.html#can_i?/3-attributes"},{"type":"function","title":"Examples - Kubereq.can_i?/3","doc":"Check for a specific action (`GET`) on a specific resource (`pods` in namespace\n`default`):\n\n    Req.new()\n    |> Kubereq.attach()\n    |> Kubereq.can_i?(verb: \"get\", version: \"v1\", resource: \"pods\", namespace: \"default\")\n\nCheck for a specific path on the API Server:\n\n    Req.new()\n    |> Kubereq.attach()\n    |> Kubereq.can_i?(verb: \"get\", path: \"apis/apiregistration.k8s.io/v1\")","ref":"Kubereq.html#can_i?/3-examples"},{"type":"function","title":"Kubereq.create/3","doc":"Create the `resource` or its `subresource` on the cluster and returns a\nresponse or an error.","ref":"Kubereq.html#create/3"},{"type":"function","title":"Example - Kubereq.create/3","doc":"Req.new()\n    |> Kubereq.attach(api_version: \"v1\", kind: \"ConfigMap\")\n    |> Kubereq.create(resource)","ref":"Kubereq.html#create/3-example"},{"type":"function","title":"Kubereq.create!/3","doc":"Create the `resource` or its `subresource` on the cluster and returns a\nresponse or raises an error.","ref":"Kubereq.html#create!/3"},{"type":"function","title":"Example - Kubereq.create!/3","doc":"Req.new()\n    |> Kubereq.attach(api_version: \"v1\", kind: \"ConfigMap\")\n    |> Kubereq.create!(resource)","ref":"Kubereq.html#create!/3-example"},{"type":"function","title":"Kubereq.delete/4","doc":"Deletes the `resource` or its `subresource` from the cluster. Returns a\nresponse or an error.","ref":"Kubereq.html#delete/4"},{"type":"function","title":"Examples - Kubereq.delete/4","doc":"Req.new()\n    |> Kubereq.attach(api_version: \"v1\", kind: \"ConfigMap\")\n    |> Kubereq.delete(\"default\", \"foo\")","ref":"Kubereq.html#delete/4-examples"},{"type":"function","title":"Kubereq.delete!/4","doc":"Deletes the `resource` or its `subresource` from the cluster. Returns a\nresponse or raises an error.","ref":"Kubereq.html#delete!/4"},{"type":"function","title":"Examples - Kubereq.delete!/4","doc":"Req.new()\n    |> Kubereq.attach(api_version: \"v1\", kind: \"ConfigMap\")\n    |> Kubereq.delete!(\"default\", \"foo\")","ref":"Kubereq.html#delete!/4-examples"},{"type":"function","title":"Kubereq.delete_all/3","doc":"Deletes all resources in the given namespace. Returns a response or an error.","ref":"Kubereq.html#delete_all/3"},{"type":"function","title":"Examples - Kubereq.delete_all/3","doc":"Req.new()\n    |> Kubereq.attach(api_version: \"v1\", kind: \"ConfigMap\")\n    |> Kubereq.delete_all(\"default\", label_selectors: [{\"app\", \"my-app\"}])","ref":"Kubereq.html#delete_all/3-examples"},{"type":"function","title":"Kubereq.delete_all!/3","doc":"Deletes all resources in the given namespace. Returns a response or raises an\nerror.","ref":"Kubereq.html#delete_all!/3"},{"type":"function","title":"Examples - Kubereq.delete_all!/3","doc":"Req.new()\n    |> Kubereq.attach(api_version: \"v1\", kind: \"ConfigMap\")\n    |> Kubereq.delete_all!(\"default\", label_selectors: [{\"app\", \"my-app\"}])","ref":"Kubereq.html#delete_all!/3-examples"},{"type":"function","title":"Kubereq.exec/4","doc":"Opens a websocket to the given Pod and executes a command on it.\nReturns a response or an error.\n\n> #### Info {: .tip}\n>\n> This function blocks the process. It should be used to execute commands\n> which terminate eventually. To implement a shell with a long running\n> connection, use `Kubereq.PodExec` with `tty: true` instead.","ref":"Kubereq.html#exec/4"},{"type":"function","title":"Examples - Kubereq.exec/4","doc":"{:ok, resp} =\n      Kubereq.exec(req, \"defaault\", \"my-pod\",\n        container: \"main-container\",\n        command: \"/bin/sh\",\n        command: \"-c\",\n        command: \"echo foobar\",\n        stdout: true,\n        stderr: true\n      )\n    Enum.each(resp.body, &IO.inspect/1)\n    # {:stdout, \"\"}\n    # {:stdout, \"foobar\\n\"}","ref":"Kubereq.html#exec/4-examples"},{"type":"function","title":"Options - Kubereq.exec/4","doc":"* `:container` (optional) - The container to connect to. Defaults to only\n  container if there is one container in the pod. Fails if not defined for\n  pods with multiple pods.\n* `:command` - Command is the remote command to execute. Not executed within a shell.\n* `:stdin` (optional) - Redirect the standard input stream of the pod for this call. Defaults to `true`.\n* `:stdin` (optional) - Redirect the standard output stream of the pod for this call. Defaults to `true`.\n* `:stderr` (optional) - Redirect the standard error stream of the pod for this call. Defaults to `true`.\n* `:tty` (optional) - If `true` indicates that a tty will be allocated for the exec call. Defaults to `false`.","ref":"Kubereq.html#exec/4-options"},{"type":"function","title":"Kubereq.exec!/4","doc":"Opens a websocket to the given Pod and executes a command on it.\nReturns a response or raises an error.\n\n> #### Info {: .tip}\n>\n> This function blocks the process. It should be used to execute commands\n> which terminate eventually. To implement a shell with a long running\n> connection, use `Kubereq.PodExec` with `tty: true` instead.","ref":"Kubereq.html#exec!/4"},{"type":"function","title":"Examples - Kubereq.exec!/4","doc":"{:ok, resp} =\n      Kubereq.exec!(req, \"defaault\", \"my-pod\",\n        container: \"main-container\",\n        command: \"/bin/sh\",\n        command: \"-c\",\n        command: \"echo foobar\",\n        stdout: true,\n        stderr: true\n      )\n    Enum.each(resp.body, &IO.inspect/1)\n    # {:stdout, \"\"}\n    # {:stdout, \"foobar\\n\"}","ref":"Kubereq.html#exec!/4-examples"},{"type":"function","title":"Options - Kubereq.exec!/4","doc":"* `:container` (optional) - The container to connect to. Defaults to only\n  container if there is one container in the pod. Fails if not defined for\n  pods with multiple pods.\n* `:command` - Command is the remote command to execute. Not executed within a shell.\n* `:stdin` (optional) - Redirect the standard input stream of the pod for this call. Defaults to `true`.\n* `:stdin` (optional) - Redirect the standard output stream of the pod for this call. Defaults to `true`.\n* `:stderr` (optional) - Redirect the standard error stream of the pod for this call. Defaults to `true`.\n* `:tty` (optional) - If `true` indicates that a tty will be allocated for the exec call. Defaults to `false`.","ref":"Kubereq.html#exec!/4-options"},{"type":"function","title":"Kubereq.get/4","doc":"Get the resource `name` in `namespace` or its `subresource`. and returns a\nresponse or an error\n\nOmit `namespace` to get cluster resources.","ref":"Kubereq.html#get/4"},{"type":"function","title":"Example - Kubereq.get/4","doc":"Req.new()\n    |> Kubereq.attach(api_version: \"v1\", kind: \"ConfigMap\")\n    |> Kubereq.get(\"default\", \"foo\")","ref":"Kubereq.html#get/4-example"},{"type":"function","title":"Kubereq.get!/4","doc":"Get the resource `name` in `namespace` or its `subresource`. and returns a\nresponse or raises an error\n\nOmit `namespace` to get cluster resources.","ref":"Kubereq.html#get!/4"},{"type":"function","title":"Example - Kubereq.get!/4","doc":"Req.new()\n    |> Kubereq.attach(api_version: \"v1\", kind: \"ConfigMap\")\n    |> Kubereq.get!(\"default\", \"foo\")","ref":"Kubereq.html#get!/4-example"},{"type":"function","title":"Kubereq.json_patch/5","doc":"Patches the resource `name`in `namespace` or its `subresource` using the given\n`json_patch`. Returns a response or an error.","ref":"Kubereq.html#json_patch/5"},{"type":"function","title":"Examples - Kubereq.json_patch/5","doc":"Req.new()\n    |> Kubereq.attach(api_version: \"v1\", kind: \"ConfigMap\")\n    |> Kubereq.json_patch(%{...}, \"default\", \"foo\")","ref":"Kubereq.html#json_patch/5-examples"},{"type":"function","title":"Kubereq.json_patch!/5","doc":"Patches the resource `name`in `namespace` or its `subresource` using the given\n`json_patch`. Returns a response or raises an error.","ref":"Kubereq.html#json_patch!/5"},{"type":"function","title":"Examples - Kubereq.json_patch!/5","doc":"Req.new()\n    |> Kubereq.attach(api_version: \"v1\", kind: \"ConfigMap\")\n    |> Kubereq.json_patch!(%{...}, \"default\", \"foo\")","ref":"Kubereq.html#json_patch!/5-examples"},{"type":"function","title":"Kubereq.list/3","doc":"Get a resource list. Returns a response or an error.","ref":"Kubereq.html#list/3"},{"type":"function","title":"Examples - Kubereq.list/3","doc":"Req.new()\n    |> Kubereq.attach(api_version: \"v1\", kind: \"ConfigMap\")\n    |> Kubereq.list(\"default\")","ref":"Kubereq.html#list/3-examples"},{"type":"function","title":"Options - Kubereq.list/3","doc":"All options described in the moduledoc plus:\n\n  * `:into` - Optional. When set to `:stream`, the underlying list request to\n    Kubernetes is paginated using `:limit` and `:continue` query parameters.\n\n  * `:limit` - Optional. Used with `into: :stream`; defines the limit query\n    parameter used for pagination.","ref":"Kubereq.html#list/3-options"},{"type":"function","title":"Async Response through the `into: :stream` - Kubereq.list/3","doc":"With `into: :srteam`, the response's `:body` is a `Stream`\n\n    {:ok, resp} =\n      Req.new()\n      |> Kubereq.attach(api_version: \"v1\", kind: \"Pod\")\n      |> Kubereq.list(into: :stream)\n    resp.body |> Stream.take(25) |> Enum.to_list()","ref":"Kubereq.html#list/3-async-response-through-the-into-stream"},{"type":"function","title":"Kubereq.list!/3","doc":"Get a resource list. Returns a response or raises an error.","ref":"Kubereq.html#list!/3"},{"type":"function","title":"Examples - Kubereq.list!/3","doc":"Req.new()\n    |> Kubereq.attach(api_version: \"v1\", kind: \"ConfigMap\")\n    |> Kubereq.list!(\"default\")","ref":"Kubereq.html#list!/3-examples"},{"type":"function","title":"Options - Kubereq.list!/3","doc":"All options described in the moduledoc plus:\n\n  * `:into` - Optional. When set to `:stream`, the underlying list request to\n    Kubernetes is paginated using `:limit` and `:continue` query parameters.\n\n  * `:limit` - Optional. Used with `into: :stream`; defines the limit query\n    parameter used for pagination.","ref":"Kubereq.html#list!/3-options"},{"type":"function","title":"Async Response through the `into: :stream` - Kubereq.list!/3","doc":"With `into: :srteam`, the response's `:body` is a `Stream`\n\n    {:ok, resp} =\n      Req.new()\n      |> Kubereq.attach(api_version: \"v1\", kind: \"Pod\")\n      |> Kubereq.list!(into: :stream)\n    resp.body |> Stream.take(25) |> Enum.to_list()","ref":"Kubereq.html#list!/3-async-response-through-the-into-stream"},{"type":"function","title":"Kubereq.logs/4","doc":"Opens a websocket to the given container and streams logs from it.\nReturns a response or an error.\n\n> #### Info {: .tip}\n>\n> This function blocks the process. It should be used to retrieve a finite\n> set of logs from a container. If you want to follow logs, use\n> `Kubereq.PodLogs` combined with the `:follow` options instead.","ref":"Kubereq.html#logs/4"},{"type":"function","title":"Examples - Kubereq.logs/4","doc":"req = Req.new() |> Kubereq.attach()\n    {:ok, resp} =\n      Kubereq.logs(req, \"default\", \"my-pod\",\n        container: \"main-container\",\n        tailLines: 5\n      )\n    Enum.each(resp.body, &IO.inspect/1)","ref":"Kubereq.html#logs/4-examples"},{"type":"function","title":"Options - Kubereq.logs/4","doc":"* `:container` - The container for which to stream logs. Defaults to only\n  container if there is one container in the pod. Fails if not defined for\n  pods with multiple pods.\n* `:follow` - Follow the log stream of the pod. If this is set to `true`,\n  the connection is kept alive which blocks current the process. If you need\n  this, you probably want to use `Kubereq.PodLogs` instead. Defaults to\n  `false`.\n* `:insecureSkipTLSVerifyBackend` - insecureSkipTLSVerifyBackend indicates\n  that the apiserver should not confirm the validity of the serving\n  certificate of the backend it is connecting to. This will make the HTTPS\n  connection between the apiserver and the backend insecure. This means the\n  apiserver cannot verify the log data it is receiving came from the real\n  kubelet. If the kubelet is configured to verify the apiserver's TLS\n  credentials, it does not mean the connection to the real kubelet is\n  vulnerable to a man in the middle attack (e.g. an attacker could not\n  intercept the actual log data coming from the real kubelet).\n* `:limitBytes` - If set, the number of bytes to read from the server before\n  terminating the log output. This may not display a complete final line of\n  logging, and may return slightly more or slightly less than the specified\n  limit.\n* `:pretty` - If 'true', then the output is pretty printed.\n* `:previous` - Return previous t  erminated container logs. Defaults to\n  `false`.\n* `:sinceSeconds` - A relative time in seconds before the current time from\n  which to show logs. If this value precedes the time a pod was started,\n  only logs since the pod start will be returned. If this value is in the\n  future, no logs will be returned. Only one of sinceSeconds or sinceTime\n  may be specified.\n* `:tailLines` - If set, the number of lines from the end of the logs to\n  show. If not specified, logs are shown from the creation of the container\n  or sinceSeconds or sinceTime\n* `:timestamps` - If true, add an RFC3339 or RFC3339Nano timestamp at the\n  beginning of every line of log output. Defaults to `false`.","ref":"Kubereq.html#logs/4-options"},{"type":"function","title":"Kubereq.logs!/4","doc":"Opens a websocket to the given container and streams logs from it.\nReturns a response or raises an error.\n\n> #### Info {: .tip}\n>\n> This function blocks the process. It should be used to retrieve a finite\n> set of logs from a container. If you want to follow logs, use\n> `Kubereq.PodLogs` combined with the `:follow` options instead.","ref":"Kubereq.html#logs!/4"},{"type":"function","title":"Examples - Kubereq.logs!/4","doc":"req = Req.new() |> Kubereq.attach()\n    {:ok, resp} =\n      Kubereq.logs!(req, \"default\", \"my-pod\",\n        container: \"main-container\",\n        tailLines: 5\n      )\n    Enum.each(resp.body, &IO.inspect/1)","ref":"Kubereq.html#logs!/4-examples"},{"type":"function","title":"Options - Kubereq.logs!/4","doc":"* `:container` - The container for which to stream logs. Defaults to only\n  container if there is one container in the pod. Fails if not defined for\n  pods with multiple pods.\n* `:follow` - Follow the log stream of the pod. If this is set to `true`,\n  the connection is kept alive which blocks current the process. If you need\n  this, you probably want to use `Kubereq.PodLogs` instead. Defaults to\n  `false`.\n* `:insecureSkipTLSVerifyBackend` - insecureSkipTLSVerifyBackend indicates\n  that the apiserver should not confirm the validity of the serving\n  certificate of the backend it is connecting to. This will make the HTTPS\n  connection between the apiserver and the backend insecure. This means the\n  apiserver cannot verify the log data it is receiving came from the real\n  kubelet. If the kubelet is configured to verify the apiserver's TLS\n  credentials, it does not mean the connection to the real kubelet is\n  vulnerable to a man in the middle attack (e.g. an attacker could not\n  intercept the actual log data coming from the real kubelet).\n* `:limitBytes` - If set, the number of bytes to read from the server before\n  terminating the log output. This may not display a complete final line of\n  logging, and may return slightly more or slightly less than the specified\n  limit.\n* `:pretty` - If 'true', then the output is pretty printed.\n* `:previous` - Return previous t  erminated container logs. Defaults to\n  `false`.\n* `:sinceSeconds` - A relative time in seconds before the current time from\n  which to show logs. If this value precedes the time a pod was started,\n  only logs since the pod start will be returned. If this value is in the\n  future, no logs will be returned. Only one of sinceSeconds or sinceTime\n  may be specified.\n* `:tailLines` - If set, the number of lines from the end of the logs to\n  show. If not specified, logs are shown from the creation of the container\n  or sinceSeconds or sinceTime\n* `:timestamps` - If true, add an RFC3339 or RFC3339Nano timestamp at the\n  beginning of every line of log output. Defaults to `false`.","ref":"Kubereq.html#logs!/4-options"},{"type":"function","title":"Kubereq.merge_patch/5","doc":"Patches the resource `name`in `namespace` or its `subresource` using the given\n`merge_patch`. Returns a response or an error.","ref":"Kubereq.html#merge_patch/5"},{"type":"function","title":"Examples - Kubereq.merge_patch/5","doc":"Req.new()\n    |> Kubereq.attach(api_version: \"v1\", kind: \"ConfigMap\")\n    |> Kubereq.merge_patch(%{...}, \"default\", \"foo\")","ref":"Kubereq.html#merge_patch/5-examples"},{"type":"function","title":"Kubereq.merge_patch!/5","doc":"Patches the resource `name`in `namespace` or its `subresource` using the given\n`merge_patch`. Returns a response or raises an error.","ref":"Kubereq.html#merge_patch!/5"},{"type":"function","title":"Examples - Kubereq.merge_patch!/5","doc":"Req.new()\n    |> Kubereq.attach(api_version: \"v1\", kind: \"ConfigMap\")\n    |> Kubereq.merge_patch!(%{...}, \"default\", \"foo\")","ref":"Kubereq.html#merge_patch!/5-examples"},{"type":"function","title":"Kubereq.new/1","doc":"","ref":"Kubereq.html#new/1"},{"type":"function","title":"Kubereq.new/2","doc":"","ref":"Kubereq.html#new/2"},{"type":"function","title":"Kubereq.update/3","doc":"Updates the given `resource`. Returns a response or an error.","ref":"Kubereq.html#update/3"},{"type":"function","title":"Examples - Kubereq.update/3","doc":"Req.new()\n    |> Kubereq.attach(api_version: \"v1\", kind: \"ConfigMap\")\n    |> Kubereq.update(resource)","ref":"Kubereq.html#update/3-examples"},{"type":"function","title":"Kubereq.update!/3","doc":"Updates the given `resource`. Returns a response or raises an error.","ref":"Kubereq.html#update!/3"},{"type":"function","title":"Examples - Kubereq.update!/3","doc":"Req.new()\n    |> Kubereq.attach(api_version: \"v1\", kind: \"ConfigMap\")\n    |> Kubereq.update!(resource)","ref":"Kubereq.html#update!/3-examples"},{"type":"function","title":"Kubereq.wait_until/5","doc":"GET a resource and wait until the given `callback` returns true or the given\n`timeout` (ms) has expired.","ref":"Kubereq.html#wait_until/5"},{"type":"function","title":"Options - Kubereq.wait_until/5","doc":"All options described in the moduledoc plus:\n\n* `:timeout` - Timeout in ms after function terminates with `{:error, :timeout}`","ref":"Kubereq.html#wait_until/5-options"},{"type":"function","title":"Kubereq.watch/3","doc":"Watch events of all resources in `namespace`. If `namespace` is `nil`, all\nnamespaces are watched. Returns a response or an error.\n\n> #### Info {: .tip}\n>\n> The Enumerable returned via the response's body blocks the process when run.\n> Use `Kubereq.Watcher` instead if you want to build a long running process\n> handling all occurring events.","ref":"Kubereq.html#watch/3"},{"type":"function","title":"Examples - Kubereq.watch/3","doc":"Req.new()\n    |> Kubereq.attach(api_version: \"v1\", kind: \"ConfigMap\")\n    |> Kubereq.watch(\"default\")\n\nOmit the `namespace` in order to watch events in all namespaces:\n\n    Req.new()\n    |> Kubereq.attach(api_version: \"v1\", kind: \"ConfigMap\")\n    |> Kubereq.watch()","ref":"Kubereq.html#watch/3-examples"},{"type":"function","title":"Options - Kubereq.watch/3","doc":"All options described in the moduledoc plus:\n\n    * `:resource_version` - Optional. Resource version to start watching from.\n      Per default, the watcher starts watching from the current\n      resource_version.","ref":"Kubereq.html#watch/3-options"},{"type":"function","title":"Kubereq.watch!/3","doc":"Watch events of all resources in `namespace`. If `namespace` is `nil`, all\nnamespaces are watched. Returns a response or raises an error.\n\n> #### Info {: .tip}\n>\n> The Enumerable returned via the response's body blocks the process when run.\n> Use `Kubereq.Watcher` instead if you want to build a long running process\n> handling all occurring events.","ref":"Kubereq.html#watch!/3"},{"type":"function","title":"Examples - Kubereq.watch!/3","doc":"Req.new()\n    |> Kubereq.attach(api_version: \"v1\", kind: \"ConfigMap\")\n    |> Kubereq.watch!(\"default\")\n\nOmit the `namespace` in order to watch events in all namespaces:\n\n    Req.new()\n    |> Kubereq.attach(api_version: \"v1\", kind: \"ConfigMap\")\n    |> Kubereq.watch!()","ref":"Kubereq.html#watch!/3-examples"},{"type":"function","title":"Options - Kubereq.watch!/3","doc":"All options described in the moduledoc plus:\n\n    * `:resource_version` - Optional. Resource version to start watching from.\n      Per default, the watcher starts watching from the current\n      resource_version.","ref":"Kubereq.html#watch!/3-options"},{"type":"function","title":"Kubereq.watch_single/4","doc":"Watch events of a single resources `name`in `namespace`. Returns a response\nor an error.","ref":"Kubereq.html#watch_single/4"},{"type":"function","title":"Examples - Kubereq.watch_single/4","doc":"Req.new()\n    |> Kubereq.attach(api_version: \"v1\", kind: \"ConfigMap\")\n    |> Kubereq.watch_single(\"default\")\n\nOmit the second argument in order to watch events in all namespaces:\n\n    Req.new()\n    |> Kubereq.attach(api_version: \"v1\", kind: \"ConfigMap\")\n    |> Kubereq.watch_single()","ref":"Kubereq.html#watch_single/4-examples"},{"type":"function","title":"Kubereq.watch_single!/4","doc":"Watch events of a single resources `name`in `namespace`. Returns a response\nor raises an error.","ref":"Kubereq.html#watch_single!/4"},{"type":"function","title":"Examples - Kubereq.watch_single!/4","doc":"Req.new()\n    |> Kubereq.attach(api_version: \"v1\", kind: \"ConfigMap\")\n    |> Kubereq.watch_single!(\"default\")\n\nOmit the second argument in order to watch events in all namespaces:\n\n    Req.new()\n    |> Kubereq.attach(api_version: \"v1\", kind: \"ConfigMap\")\n    |> Kubereq.watch_single!()","ref":"Kubereq.html#watch_single!/4-examples"},{"type":"type","title":"Kubereq.namespace/0","doc":"","ref":"Kubereq.html#t:namespace/0"},{"type":"type","title":"Kubereq.response/0","doc":"","ref":"Kubereq.html#t:response/0"},{"type":"type","title":"Kubereq.subresource/0","doc":"","ref":"Kubereq.html#t:subresource/0"},{"type":"type","title":"Kubereq.wait_until_callback/0","doc":"","ref":"Kubereq.html#t:wait_until_callback/0"},{"type":"type","title":"Kubereq.wait_until_response/0","doc":"","ref":"Kubereq.html#t:wait_until_response/0"},{"type":"module","title":"Kubereq.Access","doc":"Helper module to access maps in lists.","ref":"Kubereq.Access.html"},{"type":"function","title":"Kubereq.Access.find/2","doc":"Returns a function that accesses the first element for which `fun` returns a truthy value.\n\nThe returned function is typically passed as an accessor to `Kernel.get_in/2`,\n`Kernel.get_and_update_in/3`, and friends.","ref":"Kubereq.Access.html#find/2"},{"type":"function","title":"Examples - Kubereq.Access.find/2","doc":"iex> list = [%{name: \"john\", salary: 10}, %{name: \"francine\", salary: 30}]\n    iex> get_in(list, [Kubereq.Access.find(&(&1.name == \"john\")), :salary])\n    10\n    iex> get_and_update_in(list, [Kubereq.Access.find(&(&1.name == \"john\")), :salary], fn prev ->\n    ...>   {prev, 15}\n    ...> end)\n    {10, [%{name: \"john\", salary: 15}, %{name: \"francine\", salary: 30}]}\n\n`find/1` can also be used to pop elements out of a list or\na key inside of a list:\n\n    iex> list = [%{name: \"john\", salary: 10}, %{name: \"francine\", salary: 30}]\n    iex> pop_in(list, [Kubereq.Access.find(&(&1.name == \"francine\"))])\n    {%{name: \"francine\", salary: 30}, [%{name: \"john\", salary: 10}]}\n    iex> pop_in(list, [Kubereq.Access.find(&(&1.name == \"francine\")), :name])\n    {\"francine\", [%{salary: 30}, %{name: \"john\", salary: 10}]}\n\nWhen no match is found, the given default is used. This can be used to\nspecify defaults and safely traverse missing items.\n\n    iex> list = [%{name: \"john\", salary: 10}, %{name: \"francine\", salary: 30}]\n    iex> get_in(list, [Kubereq.Access.find(&(&1.name == \"adam\"), %{name: \"adam\", salary: 50}), :salary])\n    50\n    iex> get_and_update_in(list, [Kubereq.Access.find(&(&1.name == \"adam\"), %{name: \"adam\"}), :salary], fn prev ->\n    ...>   {prev, 50}\n    ...> end)\n    {nil, [%{name: \"adam\", salary: 50}, %{name: \"john\", salary: 10}, %{name: \"francine\", salary: 30}]}\n\nWhen multiple items exist for which `fun` return a truthy value, the first one is accessed.\n\n    iex> list = [%{name: \"john\", salary: 10}, %{name: \"john\", salary: 30}]\n    iex> get_in(list, [Kubereq.Access.find(&(&1.name == \"john\")), :salary])\n    10\n\nAn error is raised if the accessed structure is not a list:\n\n    iex> get_in(%{}, [Kubereq.Access.find(&(&1.name == \"john\"))])\n    ** (RuntimeError) Kubereq.Access.find/1 expected a list, got: %{}","ref":"Kubereq.Access.html#find/2-examples"},{"type":"function","title":"Kubereq.Access.find!/1","doc":"Returns a function that accesses the first element for which `fun` returns a truthy value.\n\nThe returned function is typically passed as an accessor to `Kernel.get_in/2`,\n`Kernel.get_and_update_in/3`, and friends.\n\nSimilar to find/2, but the returned function raises if the no item is found for which `fun` returns a truthy value.","ref":"Kubereq.Access.html#find!/1"},{"type":"function","title":"Examples - Kubereq.Access.find!/1","doc":"iex> list = [%{name: \"john\", salary: 10}, %{name: \"francine\", salary: 30}]\n    iex> get_in(list, [Kubereq.Access.find!(&(&1.name == \"john\")), :salary])\n    10\n    iex> get_and_update_in(list, [Kubereq.Access.find!(&(&1.name == \"john\")), :salary], fn prev ->\n    ...>   {prev, 15}\n    ...> end)\n    {10, [%{name: \"john\", salary: 15}, %{name: \"francine\", salary: 30}]}\n\n`find/1` can also be used to pop elements out of a list or\na key inside of a list:\n\n    iex> list = [%{name: \"john\", salary: 10}, %{name: \"francine\", salary: 30}]\n    iex> pop_in(list, [Kubereq.Access.find!(&(&1.name == \"francine\"))])\n    {%{name: \"francine\", salary: 30}, [%{name: \"john\", salary: 10}]}\n    iex> pop_in(list, [Kubereq.Access.find!(&(&1.name == \"francine\")), :name])\n    {\"francine\", [%{salary: 30}, %{name: \"john\", salary: 10}]}\n    iex> get_in(list, [Kubereq.Access.find!(&(&1.name == \"adam\")), :salary])\n    ** (ArgumentError) There is no item in the list for which the given function returns a truthy value.\n\nWhen multiple items exist for which `fun` return a truthy value, the first one is accessed.\n\n    iex> list = [%{name: \"john\", salary: 10}, %{name: \"john\", salary: 30}]\n    iex> get_in(list, [Kubereq.Access.find!(&(&1.name == \"john\")), :salary])\n    10\n\nAn error is raised if the accessed structure is not a list:\n\n    iex> get_in(%{}, [Kubereq.Access.find!(&(&1.name == \"john\"))])\n    ** (RuntimeError) Kubereq.Access.find!/1 expected a list, got: %{}","ref":"Kubereq.Access.html#find!/1-examples"},{"type":"behaviour","title":"Kubereq.Watcher","doc":"A behaviour module for implementing a Kubernetes watch event handler.\n\nEstablishes a watch connection for [efficient detection of changes]\n[k8s-watch-concept]. All events are passed to `c:handle_event/3`.\n\n[k8s-watch-concept]: https://kubernetes.io/docs/reference/using-api/api-concepts/#efficient-detection-of-changes\n\n```mermaid\nsequenceDiagram\n    participant Watcher\n    participant K8s as K8s API Server\n\n    Watcher->>K8s: ?watch=true\n\n    loop K8s Detects Changes\n      K8s ->> Watcher: {:created, %{\"kind\" => \"Pod\"}}\n      K8s ->> Watcher: {:modified, %{\"kind\" => \"Pod\"}}\n      K8s ->> Watcher: {:deleted, %{\"kind\" => \"Pod\"}}\n    end\n\n```","ref":"Kubereq.Watcher.html"},{"type":"behaviour","title":"Example - Kubereq.Watcher","doc":"When started, `Kubereq.Watcher` establishes a watch connection to the API\nServer.\n\nFor every watch event, `c:handle_event/3` is then called with the `t:type`,\n`object` and `state`.\n\n    defmodule PodEventHandler do\n      use Kubereq.Watcher\n\n      require Logger\n\n      def start_link(init_arg) do\n        req = Keyword.fetch!(init_arg, :req)\n        naemspace = Keyword.get(opts, :namespace)\n\n        Kubereq.Watcher.start_link(__MODULE__, req, namespace, api_version: \"v1\", kind: \"Pod\")\n      end\n\n      @impl Kubereq.Watcher\n      def init(init_arg) do\n        initial_state = %{}\n        {:ok, initial_state}\n      end\n\n      @impl Kubereq.Watcher\n      def handle_event(:created, pod, state) do\n        Logger.debug(\"Pod #{pod[\"metadata\"][\"name\"]} was created.\")\n        {:noreply, state}\n      end\n\n      @impl Kubereq.Watcher\n      def handle_event(:modified, pod, state) do\n        Logger.debug(\"Pod #{pod[\"metadata\"][\"name\"]} was modified.\")\n        {:noreply, state}\n      end\n\n      @impl Kubereq.Watcher\n      def handle_event(:deleted, pod, state) do\n        Logger.debug(\"Pod #{pod[\"metadata\"][\"name\"]} was deleted.\")\n        {:noreply, state}\n      end\n    end","ref":"Kubereq.Watcher.html#module-example"},{"type":"function","title":"Kubereq.Watcher.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Kubereq.Watcher.html#child_spec/1"},{"type":"function","title":"Kubereq.Watcher.connect/3","doc":"Establish a watch connection to the API Server for the given `req`.\nIf `namespace` is nil, all namespaces are watched.","ref":"Kubereq.Watcher.html#connect/3"},{"type":"function","title":"Options - Kubereq.Watcher.connect/3","doc":"All options described in `Kubereq`'s moduledoc plus:\n\n  * `:resource_version` - Optional. Resource version to start watching from.\n    Per default, the watcher starts watching from the current\n    resource_version.","ref":"Kubereq.Watcher.html#connect/3-options"},{"type":"callback","title":"Kubereq.Watcher.handle_event/3","doc":"Called for every event detected for the resources watched on the Kubernetes\ncluster. It is passed the `t:event_type` (one of `:created`, `:modified` or\n`:deleted`), the `object` (resource) the event occurred on and the current\n  `state`.","ref":"Kubereq.Watcher.html#c:handle_event/3"},{"type":"callback","title":"Kubereq.Watcher.handle_info/2","doc":"Similar to GenServer's `c:GenServer.handle_info/2`, called for messages sent\nto the process.","ref":"Kubereq.Watcher.html#c:handle_info/2"},{"type":"callback","title":"Kubereq.Watcher.init/1","doc":"Called when the server is started but before connection is establised.","ref":"Kubereq.Watcher.html#c:init/1"},{"type":"function","title":"Kubereq.Watcher.start_link/5","doc":"Starts a watcher process linked to the current process.\n\nOnce the watcher is started, the `c:init/1` function of the given module is\ncalled. After that, the watch connection is established.","ref":"Kubereq.Watcher.html#start_link/5"},{"type":"function","title":"Arguments - Kubereq.Watcher.start_link/5","doc":"`req`, `namespace` and `init_arg` are forwarded to `connect/3`.","ref":"Kubereq.Watcher.html#start_link/5-arguments"},{"type":"callback","title":"Kubereq.Watcher.terminate/2","doc":"Similar to GenServer's `c:GenServer.terminate/2`, called then the watcher is\nterminated.","ref":"Kubereq.Watcher.html#c:terminate/2"},{"type":"type","title":"Kubereq.Watcher.event_type/0","doc":"","ref":"Kubereq.Watcher.html#t:event_type/0"},{"type":"module","title":"Kubereq.PodExec","doc":"Establish a connection to a Pod and execute a command in a container.\n\nThe connection is kept alive until the websocket is closed by the counterpart.\nThe bytes received from the container are sent to the process passed via the\n`:into` option. Bytes sent to this process via `send_stdin/2` are forwarded to\nthe container.","ref":"Kubereq.PodExec.html"},{"type":"module","title":"Examples - Kubereq.PodExec","doc":"When the command terminates, the websocket is automatically closed and the\nprocess terminates.\n\n    req = Req.new() |> Kubereq.attach()\n    Kubereq.PodExec.start_link(\n      req: req,\n      namespace: \"default\",\n      name: \"my-pod\",\n      container: \"main\",\n      into: self(),\n      command: [\"/bin/sh\", \"-c\", \"echo foo\"],\n      stdin: true,\n      stdout: true,\n      stderr: true\n      tty: false,\n    )\n    # Messages in inbox: {:stdout, \"foo\\n\"}, {:close, 1000, \"\"}\n\nPassing the path to a shell as command will keep the socket open. Together\nwith `:stdin`, `:stdout`, `:stderr` and `:tty`, this can be used to implement\nan interactive shell:\n\n    req = Req.new() |> Kubereq.attach()\n    {:ok, dest} = Kubereq.PodExec.start_link(\n      req: req,\n      namespace: \"default\",\n      name: \"my-pod\",\n      container: \"main\",\n      into: self(),\n      command: [\"/bin/sh\"],\n      stdin: true,\n      stdout: true,\n      stderr: true\n      tty: false,\n    )\n    # Message in inbox: {:stdout, \"sh-5.2# \"}\n\n    Kubereq.PodExec.send_stdin(dest, \"echo foo\")\n    # Message in inbox: {:stdout, \"echo foo\\r\\nfoo\\r\\nsh-5.2# \"}","ref":"Kubereq.PodExec.html#module-examples"},{"type":"module","title":"Arguments - Kubereq.PodExec","doc":"* `:req` - A `Req.Request` struct with Kubereq attached.\n* `:namespace` - The namespace the Pod runs in\n* `:name` - The name of the Pod\n* `:container` (optional) - The container for which to stream logs. Defaults to only\n  container if there is one container in the pod. Fails if not defined for\n  pods with multiple pods.\n* `:into` - Destination for messages received from the pod. Can be a `pid` or\n  a `{pid, ref}` tuple.\n* `:command` - Command is the remote command to execute. Not executed within a shell.\n* `:stdin` (optional) - Redirect the standard input stream of the pod for this call. Defaults to `true`.\n* `:stdin` (optional) - Redirect the standard output stream of the pod for this call. Defaults to `true`.\n* `:stderr` (optional) - Redirect the standard error stream of the pod for this call. Defaults to `true`.\n* `:tty` (optional) - If `true` indicates that a tty will be allocated for the exec call. Defaults to `false`.\n* `:opts` (optional) - Additional options passed to `Req`","ref":"Kubereq.PodExec.html#module-arguments"},{"type":"function","title":"Kubereq.PodExec.close/1","doc":"Close the connection and terminate the process.","ref":"Kubereq.PodExec.html#close/1"},{"type":"function","title":"Kubereq.PodExec.close/3","doc":"","ref":"Kubereq.PodExec.html#close/3"},{"type":"function","title":"Kubereq.PodExec.open?/1","doc":"","ref":"Kubereq.PodExec.html#open?/1"},{"type":"function","title":"Kubereq.PodExec.send_stdin/2","doc":"Send the given `data` to the container.","ref":"Kubereq.PodExec.html#send_stdin/2"},{"type":"module","title":"Kubereq.PodLogs","doc":"Establish a connection to a Pod and stream logs.\n\nThe connection is kept alive until the websocket is closed via `close/1`.\nThe bytes received from the container are sent to the process passed via the\n`:into` option.","ref":"Kubereq.PodLogs.html"},{"type":"module","title":"Examples - Kubereq.PodLogs","doc":"req = Req.new() |> Kubereq.attach()\n    Kubereq.PodLogs.start_link(\n      req: req,\n      into: self(),\n      namespace: \"default\",\n      name: \"my-pod\",\n      container: \"main-container\",\n    )\n    # Messages in inbox: {:stdout, \"log entries\"}, {:stdout, \"more log entries\"}","ref":"Kubereq.PodLogs.html#module-examples"},{"type":"module","title":"Arguments - Kubereq.PodLogs","doc":"* `:req` - A `Req.Request` struct with Kubereq attached.\n* `:namespace` - The namespace the Pod runs in\n* `:name` - The name of the Pod\n* `:container` - The container for which to stream logs. Defaults to only\n  container if there is one container in the pod. Fails if not defined for\n  pods with multiple pods.\n* `:into` - Destination for messages received from the pod. Can be a `pid` or\n  a `{pid, ref}` tuple.\n* `:follow` - Follow the log stream of the pod. If this is set to `true`,\n  the connection is kept alive which blocks current the process. Defaults to\n  `true`.\n* `:insecureSkipTLSVerifyBackend` - insecureSkipTLSVerifyBackend indicates\n  that the apiserver should not confirm the validity of the serving\n  certificate of the backend it is connecting to. This will make the HTTPS\n  connection between the apiserver and the backend insecure. This means the\n  apiserver cannot verify the log data it is receiving came from the real\n  kubelet. If the kubelet is configured to verify the apiserver's TLS\n  credentials, it does not mean the connection to the real kubelet is\n  vulnerable to a man in the middle attack (e.g. an attacker could not\n  intercept the actual log data coming from the real kubelet).\n* `:limitBytes` - If set, the number of bytes to read from the server before\n  terminating the log output. This may not display a complete final line of\n  logging, and may return slightly more or slightly less than the specified\n  limit.\n* `:pretty` - If 'true', then the output is pretty printed.\n* `:previous` - Return previous terminated container logs. Defaults to `false`.\n* `:sinceSeconds` - A relative time in seconds before the current time from\n  which to show logs. If this value precedes the time a pod was started,\n  only logs since the pod start will be returned. If this value is in the\n  future, no logs will be returned. Only one of sinceSeconds or sinceTime\n  may be specified.\n* `:tailLines` - If set, the number of lines from the end of the logs to\n  show. If not specified, logs are shown from the creation of the container\n  or sinceSeconds or sinceTime\n* `:timestamps` - If true, add an RFC3339 or RFC3339Nano timestamp at the\n  beginning of every line of log output. Defaults to `false`.\n* `:opts` (optional) - Additional options passed to `Req`","ref":"Kubereq.PodLogs.html#module-arguments"},{"type":"function","title":"Kubereq.PodLogs.close/1","doc":"Close the connection and terminate the process.","ref":"Kubereq.PodLogs.html#close/1"},{"type":"function","title":"Kubereq.PodLogs.close/3","doc":"","ref":"Kubereq.PodLogs.html#close/3"},{"type":"function","title":"Kubereq.PodLogs.open?/1","doc":"","ref":"Kubereq.PodLogs.html#open?/1"},{"type":"module","title":"Kubereq.Kubeconfig","doc":"This is the `Pluggable.Token` for the pipeline loading the Kubernetes config.\nThe Kubeconfig represents the configuration to establish a connection to\nthe Kubernetes cluster. It contains informations like endpoint, certificates,\nuser authentication details etc.\n\nIn most cases you can just rely on `Kubereq.Kubeconfig.Default` to load\nthe Kubeconfig from well-known places.\n\nSometimes you only want to allow to load the Kubeconfig from a specific\nYAML file or rely on an ENV variable pointing to that file. Check out\nthe `Kubereq.Kubeconfig.*` modules.\n\nYou can also chain these modules to build your own Kubeconfig loader pipeline.\n\n```\ndefmodule MyKubeconfLoader do\n  use Pluggable.StepBuilder\n\n  step Kubereq.Kubeconfig.ENV\n  step Kubereq.Kubeconfig.File, path: \"/path/to/kubeconfig.yaml\"\nend\n```","ref":"Kubereq.Kubeconfig.html"},{"type":"function","title":"Kubereq.Kubeconfig.load/1","doc":"Loads the Kubernetes config by running the given `pipeline`. Returns the\nresulting `%Kubereq.Kubeconfig{}`.\n\n`pipeline` can be passed in the form of `{pipeline_module, opts}` tuples,\na single `pipeline_module` or a list of either.","ref":"Kubereq.Kubeconfig.html#load/1"},{"type":"function","title":"Example - Kubereq.Kubeconfig.load/1","doc":"Single pipeline module without opts passed as module:\n\n    Kubereq.Kubeconfig.load(Kubereq.Kubeconfig.Default)\n\nSingle pipeline module with opts:\n\n    Kubereq.Kubeconfig.load({Kubereq.Kubeconfig.File, path: \"/path/to/kubeconfig\"})\n\nList of either:\n\n    Kubereq.Kubeconfig.load([\n      Kubereq.Kubeconfig.ENV,\n      {Kubereq.Kubeconfig.File, path: \"~/.kube/config\"},\n      Kubereq.Kubeconfig.ServiceAccount\n    ])","ref":"Kubereq.Kubeconfig.html#load/1-example"},{"type":"function","title":"Kubereq.Kubeconfig.new!/1","doc":"Creates a new `%Kubereq.Kubeconfig{}` struct with the given fields","ref":"Kubereq.Kubeconfig.html#new!/1"},{"type":"function","title":"Kubereq.Kubeconfig.set_current_context/2","doc":"Sets the current context. This function sets `current_cluster` and\n`current_user` in the given `Kubereq.Kubeconfig.t()`","ref":"Kubereq.Kubeconfig.html#set_current_context/2"},{"type":"type","title":"Kubereq.Kubeconfig.t/0","doc":"The `%Kubereq.Kubeconfig{}` struct holds information required to connect to\nKubernetes clusters\n\nFor descriptions of the fields, refer to the\n[kubeconfig.v1](https://kubernetes.io/docs/reference/config-api/kubeconfig.v1/) documentation.","ref":"Kubereq.Kubeconfig.html#t:t/0"},{"type":"module","title":"Kubereq.Kubeconfig.Default","doc":"Default pipeline of pluggable steps for loading the Kubeconfig. Tries to\nbuild the Kubeconfig from different default settings.\n\n1. Checks for the `KUBECONFIG` environment variable. If it is set and pointing\n   to a Kubeconfig file, that file is imported.\n1. Checks for `$HOME/.kube/config`.\n1. Checks if running inside a Cluster and tries to connect using the Service\n   Account Token.","ref":"Kubereq.Kubeconfig.Default.html"},{"type":"module","title":"Example - Kubereq.Kubeconfig.Default","doc":"Usage in a pipeline created with `Pluggable.StepBuilder`:\n\n    defmodule MyApp.KubeconfLoader do\n      use Pluggable.StepBuilder\n\n      step Kubereq.Kubeconfig.Default\n    end","ref":"Kubereq.Kubeconfig.Default.html#module-example"},{"type":"module","title":"Kubereq.Kubeconfig.ENV","doc":"Pluggable step that loads the Kubeconfig from a config file whose location is\ndefined by an ENV variable. Uses `Kubereq.Kubeconfig.File` under the hood.\n\n### Examples\n\n    step Kubereq.Kubeconfig.ENV\n\nBy default, this step assumes the name of the variable to be `KUBECONFIG`.\nThis can be customized through the `:env_var` option\n\n    step Kubereq.Kubeconfig.ENV, env_var: SPECIAL_KUBECONFIG\n\n### Options\n\n* `env_var` - (optional) The name of the environment variable. Defaults to\n  `KUBECONFIG`\n* `!` - (optional. And yes, that's a valid atom) Raise an exception if the\n  env var not found. Defaults to `false`.","ref":"Kubereq.Kubeconfig.ENV.html"},{"type":"module","title":"Kubereq.Kubeconfig.File","doc":"Pluggable step that load the Kubeconfig from a file.\n\n```\nstep Kubereq.Kubeconfig.File, path: \"path/to/kubeconfig-integration.yaml\"\n```\n\nIf the config file defined by the `:path` option is not found on disk, by\ndefault, the step gracefully returns the kubeconfig that was passed as\nargument. If you want the step to raise an `ArgumentException` instead, you\ncan set the option `!: true`.\n\n### Options\n\n* `:path` - Path to the config file.\n* `:!` - (optional. And yes, that's a valid atom) Raise an exception if the\n  config file is not found. Defaults to `false`.\n* `:context` - (optional) Sets the current context in case there's multiple\n  contexts defined in the config file. Defaults to what's defined in the\n  \"current-context\" field in the loaded config.","ref":"Kubereq.Kubeconfig.File.html"},{"type":"module","title":"Kubereq.Kubeconfig.ServiceAccount","doc":"Pluggable step that builds the Kubeconfig using a Service Account's token for\nauthentication.\n\nWhen running the app inside a Kubernetes cluster, make sure RBAC is configured\ncorrectly and use this step. It reads the service account's JWC token and\nbuild the Kubeconfig accordingly.","ref":"Kubereq.Kubeconfig.ServiceAccount.html"},{"type":"module","title":"Examples - Kubereq.Kubeconfig.ServiceAccount","doc":"step Kubereq.Kubeconfig.ServiceAccount\n\nIf your token is mounted at a different location than the default, pass its\nlocation as `:path_to_folder`.\n\n    step Kubereq.Kubeconfig.ServiceAccount, path_to_folder: \"path/to/folder/with/token\"","ref":"Kubereq.Kubeconfig.ServiceAccount.html#module-examples"},{"type":"module","title":"Options - Kubereq.Kubeconfig.ServiceAccount","doc":"* `path_to_folder` - (optional) Path to the folder where the `token`, `ca.crt`\n  and `namespace` files of the service account are located. Defaults to\n  `\"/var/run/secrets/kubernetes.io/serviceaccount\"`\n* `:!` - (optional. And yes, that's a valid atom) Raise an exception if the\n  config file is not found. Defaults to `false`.","ref":"Kubereq.Kubeconfig.ServiceAccount.html#module-options"},{"type":"module","title":"Kubereq.Kubeconfig.Stub","doc":"Req testing conveniences for `kubereq` requests.\n\nSince `kubereq` is using `Req` under the hood, we can use\n[`Req.Test`](https://hexdocs.pm/req/Req.Test.html) to run requests through\nmocks/stubs. Using this step as your Kubeconfig pipeline, you can set a stub\non the `Req.Request` configured by this step.\n\nIn your tests, you can then use `Req.Test` according to its documentation.","ref":"Kubereq.Kubeconfig.Stub.html"},{"type":"module","title":"Example - Kubereq.Kubeconfig.Stub","doc":"Imagine we're building an app with a pod client that lists pods on the\ncluster.\n\nWe start off by defining a module for loading the Kubernetes config:\n\n    defmodule MyApp.Kubeconfig do\n      @kubeconfig_pipeline Application.compile_env(:myapp, :kubeconfig_pipeline)\n\n      def load(), do: Kubereq.Kubeconfig.load(@kubeconfig_pipeline)\n    end\n\nWe then implement the pod client using the Kubeconfig loader to create a\n`Req.Request`:\n\n    defmodule MyApp.PodClient do\n      @resource_path \"api/v1/namespaces/:namespace/pods/:name\"\n\n      def list(namespace) do\n        req = Kubereq.new(MyApp.Kubeconfig.load(), @resource_path)\n\n        {:ok, resp} = Kubereq.list(req, namespace)\n        resp.body[\"items\"]\n      end\n    end\n\nWe configure the kubeconfig pipeline for production using\n`Kubereq.Kubeconfig.Default` as our pipeline to load the Kubeconfig:\n\n    # config/prod.exs\n    config :myapp, kubeconfig_pipeline: Kubereq.Kubeconfig.Default\n\nIn tests, instead of sending requests to the cluster, we make the request\nagainst a plug stub named `MyApp.Cluster`:\n\n    # config/test.exs\n    config :myapp, kubeconfig_pipeline: {Kubereq.Kubeconfig.Stub, plugs: {Req.Test, MyApp.Cluster}}\n\nNow we can control our stubs in concurrent tests:\n\n    use ExUnit.Case, async: true\n\n    test \"many pods\" do\n      Req.Test.stub(MyApp.Cluster, fn conn ->\n        Req.Test.json(conn, %{\n          \"apiVersion\" => \"v1\",\n          \"kind\" => \"List\",\n          \"items\" => [\n            %{\n              \"apiVersion\" => \"v1\",\n              \"kind\" => \"Pod\"\n              # ...\n            }\n          ]\n        })\n      end)\n\n      assert [_] = MyApp.PodClient.list(\"default\")\n    end","ref":"Kubereq.Kubeconfig.Stub.html#module-example"},{"type":"module","title":"Stubs per Context - Kubereq.Kubeconfig.Stub","doc":"If you want to simulate multiple different clusters or different responses for\nthe same calls, you can pass a `%{context :: String.t() => plug :: tuple}` map\nto as `plugs` option.\n\n    # config/test.exs\n    config :myapp, kubeconfig_pipeline: {Kubereq.Kubeconfig.Stub, plugs: %{\n      \"happy-path\" => {Req.Test, MyApp.HappyPathCluster}\n      \"missing-permissions\" => {Req.Test, MyApp.MissingPermissionsCluster}\n    }}\n\n## Options\n\n* `plugs` - The plug or `%{context => plug}` map to be configured on the\n  `Req.Request` configured by this step.","ref":"Kubereq.Kubeconfig.Stub.html#module-stubs-per-context"},{"type":"module","title":"Kubereq.Step.FieldSelector","doc":"Req step to format field selectors.\n\nField selectors let you select Kubernetes objects based on the value of one or more resource fields.\nThe concept is explained on the Kubernetes documentation\n[Field Selectors](https://kubernetes.io/docs/concepts/overview/working-with-objects/field-selectors/)\n\nThe functions for listing and watching resources accept an option\n`:field_selectors` as a list of strings or tuples.","ref":"Kubereq.Step.FieldSelector.html"},{"type":"module","title":"Examples - Kubereq.Step.FieldSelector","doc":"The following are equivalent field selectors for equality:\n\n* `\"metadata.name=my-service\"`\n* `\"metadata.name==my-service\"`\n* `{\"metadata.name\", \"my-service\"}\n* `{\"metadata.name\", {:eq, \"my-service\"}}\n\nThe following are equivalent field selectors for inequality:\n\n* `\"metadata.namespace!=default\"`\n* `{\"metadata.namespace\", {:neq, \"default\"}}`\n* `{\"metadata.namespace\", {:ne, \"default\"}}`","ref":"Kubereq.Step.FieldSelector.html#module-examples"},{"type":"function","title":"Kubereq.Step.FieldSelector.call/1","doc":"","ref":"Kubereq.Step.FieldSelector.html#call/1"},{"type":"module","title":"Kubereq.Step.LabelSelector","doc":"Req step to format label selectors.\n\nLabel selectors are used to filter list and watch operations by resource\nlabels. The concept is explained on the Kubernetes\ndocumentation [Labels and Selectors](https://kubernetes.io/docs/concepts/overview/working-with-objects/labels)\n\nThe functions for listing and watching resources accept an option\n`:label_selectors` as a list of strings or tuples.","ref":"Kubereq.Step.LabelSelector.html"},{"type":"module","title":"Equality-based requirements - Kubereq.Step.LabelSelector","doc":"The following are equivalent label selectors for equality:\n\n* `\"environment = production\"`\n* `{\"environment\", \"production\"}`\n* `{\"environment\", {:eq, \"production\"}}`\n\nThe following are equivalent label selectors for inequality:\n\n* `\"tier != frontend\"`\n* `{\"tier\", {:neq, \"frontend\"}}`\n* `{\"tier\", {:ne, \"frontend\"}}`","ref":"Kubereq.Step.LabelSelector.html#module-equality-based-requirements"},{"type":"module","title":"Set-based requirements - Kubereq.Step.LabelSelector","doc":"The following are equivalent label selectors for `In` requirements:\n\n* `\"environment in (production, qa)\"`\n* `{\"environment\", [\"production\", \"qa\"]}`\n* `{\"environment\", {:in, [\"production\", \"qa\"]}}`\n\nThe following are equivalent label selectors for `NotIn` requirements:\n\n* `\"tier notin (frontend, backend)\"`\n* `{\"tier\", {:notin, [\"frontend\", \"backend\"]}}`\n* `{\"tier\", {:not_in, [\"frontend\", \"backend\"]}}`\n\nThe following are equivalent label selectors for `Exists` requirements:\n\n* `\"partition\"`\n* `{\"partition\"}`\n* `{\"partition\", :exists}`\n\nThe following are equivalent label selectors for `not DoesNotExist` requirements:\n\n* `\"!partition\"`\n* `{\"!partition\"}`\n* `{\"partition\", :notexists}`\n* `{\"partition\", :not_exists}`","ref":"Kubereq.Step.LabelSelector.html#module-set-based-requirements"},{"type":"function","title":"Kubereq.Step.LabelSelector.call/1","doc":"","ref":"Kubereq.Step.LabelSelector.html#call/1"},{"type":"exception","title":"Kubereq.Error.KubeconfError","doc":"Indicates an error with the Kubernetes Configuration","ref":"Kubereq.Error.KubeconfError.html"},{"type":"function","title":"Kubereq.Error.KubeconfError.new/2","doc":"","ref":"Kubereq.Error.KubeconfError.html#new/2"},{"type":"type","title":"Kubereq.Error.KubeconfError.t/0","doc":"","ref":"Kubereq.Error.KubeconfError.html#t:t/0"},{"type":"exception","title":"Kubereq.Error.StepError","doc":"Indicates an error during the Req steps processing.","ref":"Kubereq.Error.StepError.html"},{"type":"function","title":"Kubereq.Error.StepError.new/2","doc":"","ref":"Kubereq.Error.StepError.html#new/2"},{"type":"type","title":"Kubereq.Error.StepError.t/0","doc":"","ref":"Kubereq.Error.StepError.html#t:t/0"},{"type":"extras","title":"Kubereq","doc":"# Kubereq\n\nUsed by [`kubegen`](https://github.com/mruoss/kubegen) to build Resource based\nKubernetes API clients using Req with `kubereq`.\n\n[![Module Version](https://img.shields.io/hexpm/v/kubereq.svg)](https://hex.pm/packages/kubereq)\n[![Last Updated](https://img.shields.io/github/last-commit/mruoss/kubereq.svg)](https://github.com/mruoss/kubereq/commits/main)\n\n[![Hex Docs](https://img.shields.io/badge/hex-docs-lightgreen.svg)](https://hexdocs.pm/kubereq/)\n[![Total Download](https://img.shields.io/hexpm/dt/kubereq.svg)](https://hex.pm/packages/kubereq)\n[![License](https://img.shields.io/hexpm/l/kubereq.svg)](https://github.com/mruoss/kubereq/blob/main/LICENSE.md)\n\nWhile this library can be used directly, it is easier to let\n[`kubegen`](https://github.com/mruoss/kubegen) generate the API client modules\nfor you. The resulting clients are then using `kubereq` to get the prepared\n`Req.Request` struct and make the requests to the Kubernetes API Server.","ref":"readme.html"},{"type":"extras","title":"Installation - Kubereq","doc":"The package can be installed by adding `kubereq` to your list of dependencies in\n`mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:kubereq, \"~> 0.4.0\"}\n  ]\nend\n```\n\nThe docs can be found at  .","ref":"readme.html#installation"},{"type":"extras","title":"Usage - Kubereq","doc":"This library can be used with plain `Req` but the functions in this module\nprovide an easier API to people used to `kubectl` and friends.","ref":"readme.html#usage"},{"type":"extras","title":"Plain Req - Kubereq","doc":"Use `Kubereq.Kubeconfig.Default` to create connection to cluster and\nplain `Req.request()` to make the request\n\n```ex\nreq = Req.new() |> Kubereq.attach()\n\nReq.request!(req,\n  api_version: \"v1\",\n  kind: \"ServiceAccount\",\n  operation: :get,\n  path_params: [namespace: \"default\", name: \"default\"]\n)\n```\n\nYou can pass your own Kubeconfigloader pipeline when attaching:\n\n```ex\nreq = Req.new() |> Kubereq.attach(kubeconfig: {Kubereq.Kubeconfig.File, path: \"/path/to/kubeconfig.yaml\"})\n\nReq.request!(req,\n  api_version: \"v1\",\n  kind: \"ServiceAccount\",\n  operation: :get,\n  path_params: [namespace: \"default\", name: \"default\"]\n)\n```\n\nPrepare a `Req` struct for a specific resource:\n\n```ex\nsa_req = Req.new() |> Kubereq.attach(api_version: \"v1\", kind: \"ServiceAccount\")\n\nReq.request!(sa_req,  operation: :get, path_params: [namespace: \"default\", name: \"default\"])\nReq.request!(sa_req,  operation: :list, path_params: [namespace: \"default\"])\n```","ref":"readme.html#plain-req"},{"type":"extras","title":"Kubereq API - Kubereq","doc":"While this library can attach to any `Req` struct, it is sometimes easier\nto prepare `Req` for a specific resource and then use the functions\ndefined in the `Kubereq` module.\n\n```ex\nsa_req = Req.new() |> Kubereq.attach(api_version: \"v1\", kind: \"ServiceAccount\")\n\nKubereq.get(sa_req, \"my-namespace\", \"default\")\nKubereq.list(sa_req, \"my-namespace\")\n```\n\nOr use the functions right away, defining the resource through options:\n\n```ex\nreq = Req.new() |> Kubereq.attach()\n\nKubereq.get(req, \"my-namespace\", \"default\", api_version: \"v1\", kind: \"ServiceAccount\")\n\n# get the \"status\" subresource of the default namespace\nKubereq.get(req, \"my-namespace\", api_version: \"v1\", kind: \"Namespace\", subresource: \"status\")\n```\n\nFor resources defined by Kubernetes, the `api_version` can be omitted:\n\n```ex\nReq.new()\n|> Kubereq.attach(kind: \"Namespace\")\n|> Kubereq.get(\"my-namespace\")\n```","ref":"readme.html#kubereq-api"},{"type":"extras","title":"Changelog","doc":"# Changelog\n\nAll notable changes to this project will be documented in this file.\n\nThe format is based on [Keep a Changelog](https://keepachangelog.com/en/1.0.0/),\nand this project adheres to [Semantic Versioning](https://semver.org/spec/v2.0.0.html).","ref":"changelog.html"},{"type":"extras","title":"Unreleased - Changelog","doc":"<!-- Add your changelog entry to the relevant subsection -->\n\n<!--","ref":"changelog.html#unreleased"},{"type":"extras","title":"Added | Changed | Deprecated | Removed | Fixed | Security --> - Changelog","doc":"<!--------------------- Don't add new entries after this line --------------------->","ref":"changelog.html#added-changed-deprecated-removed-fixed-security"},{"type":"extras","title":"0.4.0 - 2024-12-13 - Changelog","doc":"","ref":"changelog.html#0-4-0-2024-12-13"},{"type":"extras","title":"⚠️ Breaking Changes ⚠️ - Changelog","doc":"- `Kubereq.watch/3` and `Kubereq.watch_single/4` now also return `t:Kubereq.response/0`.\n  The stream which was returned directly before is now accessible through the `:body`\n  key of the returned `t:Req.Response.t/0` struct. The `:stream_to` option was removed.\n  `Kubereq.Watcher` was added for implementing long running watch event handlers.","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"Changes - Changelog","doc":"- `Kubereq.wait_until/5` gets by without `Task.async/1`.\n- `Kubereq.create/3`, `Kubereq.update/3` and `Kubereq.apply/5`: Extract `api_verison` and `kind` from passed resource.","ref":"changelog.html#changes"},{"type":"extras","title":"Added - Changelog","doc":"- `Kubereq.Watcher` - A behaviour module for implementing a Kubernetes watch event handler.\n- Streaming for paginated list operation via the `:into` option of `Kubereq.list/3`\n  [#46](https://github.com/mruoss/kubereq/pull/46)\n- `Kubereq.can_i?/3` - Checks whether the authenticated user is authorized to perform a specific action.\n- Error raising variants for all functions in `Kubereq` except `Kubereq.wait_until/5`.","ref":"changelog.html#added"},{"type":"extras","title":"0.3.2 - 2024-11-22 - Changelog","doc":"","ref":"changelog.html#0-3-2-2024-11-22"},{"type":"extras","title":"Added - Changelog","doc":"- Support for websocket connections to `pods/log` and `pods/exec` subresources [#37](https://github.com/mruoss/kubereq/pull/37)","ref":"changelog.html#added"},{"type":"extras","title":"0.3.1 - 2024-10-24 - Changelog","doc":"","ref":"changelog.html#0-3-1-2024-10-24"},{"type":"extras","title":"Fixed - Changelog","doc":"- Include resource name in the path for deletion in order to prevent deleting multiple resources [#41](https://github.com/mruoss/kubereq/issues/41), [#42](https://github.com/mruoss/kubereq/pull/42)","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- Add current context's namespace as `:current_namespace` field to Kubeconfig. [#39](https://github.com/mruoss/kubereq/pull/39)\n- Add Req option `:context` to switch context ad-hoc. [#39](https://github.com/mruoss/kubereq/pull/39)","ref":"changelog.html#added"},{"type":"extras","title":"0.3.0 - 2024-10-04 - Changelog","doc":"","ref":"changelog.html#0-3-0-2024-10-04"},{"type":"extras","title":"⚠️ Breaking ⚠️ - Changelog","doc":"This release comes with a refactored API of the `kubereq` library and will not\nwork for code using earlier versions of `kubereq`. Migrating should be straight\nforward in most cases.\n\n- `Kubereq.new/N` was deprecated and replaced with `Kubereq.attach/1`\n- The result of `Kuberq.attach/1` can be used with `Req` functions directly\n- Functions in the `Kubereq` modules still provide a nice abstraction over plain\n  `Req`. They now forward all `opts` to `Req`.\n- `Kubereq.wait_until/5` now takes a Keyword list as fifth argument (was\n  `integer` before). To migrate, just turn `timeout` into `timeout: timeout`","ref":"changelog.html#breaking"},{"type":"extras","title":"0.2.1 - 2024-10-03 - Changelog","doc":"","ref":"changelog.html#0-2-1-2024-10-03"},{"type":"extras","title":"Fixed - Changelog","doc":"- Exec auth wrongly assumes cert data to be base64 encoded [#33](https://github.com/mruoss/kubereq/issues/33), [#34](https://github.com/mruoss/kubereq/pull/34)","ref":"changelog.html#fixed"},{"type":"extras","title":"0.2.0 - 2024-09-20 - Changelog","doc":"","ref":"changelog.html#0-2-0-2024-09-20"},{"type":"extras","title":"Changed - Changelog","doc":"- `exec` auth and other steps now return errors instead of raising exceptions. [#30](https://github.com/mruoss/kubereq/pull/30)","ref":"changelog.html#changed"},{"type":"extras","title":"0.1.8 - 2024-09-14 - Changelog","doc":"","ref":"changelog.html#0-1-8-2024-09-14"},{"type":"extras","title":"Changed - Changelog","doc":"- `Kubereq.Kubeconfig.Stub`: Set server url to context name","ref":"changelog.html#changed"},{"type":"extras","title":"0.1.7 - 2024-09-12 - Changelog","doc":"","ref":"changelog.html#0-1-7-2024-09-12"},{"type":"extras","title":"Added - Changelog","doc":"- `Kubereq.Kubeconfig.Stub`: A Kubeconfig step used for testing","ref":"changelog.html#added"},{"type":"extras","title":"0.1.6 - 2024-08-25 - Changelog","doc":"","ref":"changelog.html#0-1-6-2024-08-25"},{"type":"extras","title":"Fixed - Changelog","doc":"- `Kubereq.Kubeconfig.ServiceAccount`: make server key a binary","ref":"changelog.html#fixed"},{"type":"extras","title":"0.1.5 - 2024-08-24 - Changelog","doc":"","ref":"changelog.html#0-1-5-2024-08-24"},{"type":"extras","title":"Added - Changelog","doc":"- `Kubereq.wait_until/5`: Support returning `{:error, error}` tuple in the callback","ref":"changelog.html#added"},{"type":"extras","title":"0.1.4 - 2024-08-21 - Changelog","doc":"","ref":"changelog.html#0-1-4-2024-08-21"},{"type":"extras","title":"Fixed - Changelog","doc":"- `Kubereq.Kubeconfig.File`: Only expand path if not relative to HOME.","ref":"changelog.html#fixed"},{"type":"extras","title":"0.1.3 - 2024-08-19 - Changelog","doc":"","ref":"changelog.html#0-1-3-2024-08-19"},{"type":"extras","title":"Fixed - Changelog","doc":"- Fixed several bugs and docs\n- Dependency updates","ref":"changelog.html#fixed"},{"type":"extras","title":"0.1.2 - 2024-06-13 - Changelog","doc":"- Dependency updates","ref":"changelog.html#0-1-2-2024-06-13"},{"type":"extras","title":"0.1.1 - Changelog","doc":"","ref":"changelog.html#0-1-1"},{"type":"extras","title":"Fixed - Changelog","doc":"- License was wrong in `mix.exs`","ref":"changelog.html#fixed"},{"type":"extras","title":"Unreleased - Changelog","doc":"<!-- Add your changelog entry to the relevant subsection -->\n\n<!--","ref":"changelog.html#unreleased"},{"type":"extras","title":"Added | Changed | Deprecated | Removed | Fixed | Security --> - Changelog","doc":"<!-- No new entries below this line! -->","ref":"changelog.html#added-changed-deprecated-removed-fixed-security"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.35.0"}}